@@ -1,9 +1,9 @@
 package Net::DNS::Packet;
 
-#
-# $Id: Packet.pm 1747 2019-06-27 15:02:31Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1747 $)[1];
+use strict;
+use warnings;
+
+our $VERSION = (qw$Id: Packet.pm 1812 2020-10-07 18:09:53Z willem $)[2];
 
 
 =head1 NAME
@@ -14,7 +14,7 @@ Net::DNS::Packet - DNS protocol packet
 
     use Net::DNS::Packet;
 
-    $query = new Net::DNS::Packet( 'example.com', 'MX', 'IN' );
+    $query = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
 
     $reply = $resolver->send( $query );
 
@@ -26,11 +26,10 @@ A Net::DNS::Packet object represents a DNS protocol packet.
 =cut
 
 
-use strict;
-use warnings;
 use integer;
 use Carp;
 
+use Net::DNS::Parameters qw(:dsotype);
 use constant UDPSZ => 512;
 
 BEGIN {
@@ -44,10 +43,10 @@ BEGIN {
 
 =head2 new
 
-    $packet = new Net::DNS::Packet( 'example.com' );
-    $packet = new Net::DNS::Packet( 'example.com', 'MX', 'IN' );
+    $packet = Net::DNS::Packet->new( 'example.com' );
+    $packet = Net::DNS::Packet->new( 'example.com', 'MX', 'IN' );
 
-    $packet = new Net::DNS::Packet();
+    $packet = Net::DNS::Packet->new();
 
 If passed a domain, type, and class, new() creates a Net::DNS::Packet
 object which is suitable for making a DNS query for the specified
@@ -80,8 +79,9 @@ sub new {
 
 =pod
 
-    $packet = new Net::DNS::Packet( \$data );
-    $packet = new Net::DNS::Packet( \$data, 1 );	# debug
+    $packet = Net::DNS::Packet->decode( \$data );
+    $packet = Net::DNS::Packet->decode( \$data, 1 );	# debug
+    $packet = Net::DNS::Packet->new( \$data ... );
 
 If passed a reference to a scalar containing DNS packet data, a new
 packet object is created by decoding the data.
@@ -93,7 +93,7 @@ Decoding errors, including data corruption and truncation, are
 collected in the $@ ($EVAL_ERROR) variable.
 
 
-    ( $packet, $length ) = new Net::DNS::Packet( \$data );
+    ( $packet, $length ) = Net::DNS::Packet->decode( \$data );
 
 If called in array context, returns a packet object and the number
 of octets successfully decoded.
@@ -120,7 +120,7 @@ sub decode {
 		# header section
 		my ( $id, $status, @count ) = unpack 'n6', $$data;
 		my ( $qd, $an, $ns, $ar ) = @count;
-		$offset = HEADER_LENGTH;
+		my $length = length $$data;
 
 		$self = bless {
 			id	   => $id,
@@ -130,12 +130,13 @@ sub decode {
 			answer	   => [],
 			authority  => [],
 			additional => [],
-			replysize  => length $$data
+			replysize  => $length
 			}, $class;
 
 		# question/zone section
 		my $hash = {};
 		my $record;
+		$offset = HEADER_LENGTH;
 		while ( $qd-- ) {
 			( $record, $offset ) = decode Net::DNS::Question( $data, $offset, $hash );
 			CORE::push( @{$self->{question}}, $record );
@@ -157,7 +158,15 @@ sub decode {
 			CORE::push( @{$self->{additional}}, $record );
 		}
 
-		return $self;
+		return unless $offset == HEADER_LENGTH;
+		return unless $self->header->opcode eq 'DSO';
+
+		$self->{dso} = [];
+		while ( $offset < $length ) {
+			my ( $t, $l, $v ) = unpack "\@$offset n2a*", $$data;
+			CORE::push( @{$self->{dso}}, [$t, substr( $v, 0, $l )] );
+			$offset += 4 + $l;
+		}
 	};
 
 	if ($debug) {
@@ -183,25 +192,25 @@ Truncation may be specified using a non-zero optional size argument.
 =cut
 
 sub data {
-	&encode;
+	return &encode;
 }
 
 sub encode {
 	my ( $self, $size ) = @_;				# uncoverable pod
 
 	my $edns = $self->edns;					# EDNS support
-	my @addl = grep !$_->isa('Net::DNS::RR::OPT'), @{$self->{additional}};
+	my @addl = grep { !$_->isa('Net::DNS::RR::OPT') } @{$self->{additional}};
 	$self->{additional} = [$edns, @addl] if $edns->_specified;
 
 	return $self->truncate($size) if $size;
 
 	my @part = qw(question answer authority additional);
-	my @size = map scalar( @{$self->{$_}} ), @part;
+	my @size = map { scalar @{$self->{$_}} } @part;
 	my $data = pack 'n6', $self->header->id, $self->{status}, @size;
 	$self->{count} = [];
 
 	my $hash = {};						# packet body
-	foreach my $component ( map @{$self->{$_}}, @part ) {
+	foreach my $component ( map { @{$self->{$_}} } @part ) {
 		$data .= $component->encode( length $data, $hash, $self );
 	}
 
@@ -220,7 +229,7 @@ represents the header section of the packet.
 
 sub header {
 	my $self = shift;
-	bless \$self, q(Net::DNS::Header);
+	return bless \$self, q(Net::DNS::Header);
 }
 
 
@@ -238,8 +247,8 @@ extension OPT RR.
 sub edns {
 	my $self = shift;
 	my $link = \$self->{xedns};
-	($$link) = grep $_->isa(qw(Net::DNS::RR::OPT)), @{$self->{additional}} unless $$link;
-	$$link = new Net::DNS::RR( type => 'OPT' ) unless $$link;
+	($$link) = grep { $_->isa(qw(Net::DNS::RR::OPT)) } @{$self->{additional}} unless $$link;
+	$$link = Net::DNS::RR->new( type => 'OPT' ) unless $$link;
 	return $$link;
 }
 
@@ -262,7 +271,7 @@ sub reply {
 	my $qheadr = $query->header;
 	croak 'erroneous qr flag in query packet' if $qheadr->qr;
 
-	my $reply  = new Net::DNS::Packet();
+	my $reply  = Net::DNS::Packet->new();
 	my $header = $reply->header;
 	$header->qr(1);						# reply with same id, opcode and question
 	$header->id( $qheadr->id );
@@ -275,7 +284,7 @@ sub reply {
 	$header->rd( $qheadr->rd );				# copy these flags into reply
 	$header->cd( $qheadr->cd );
 
-	return $reply unless grep $_->isa('Net::DNS::RR::OPT'), @{$query->{additional}};
+	return $reply unless grep { $_->isa('Net::DNS::RR::OPT') } @{$query->{additional}};
 
 	my $edns = $reply->edns();
 	CORE::push( @{$reply->{additional}}, $edns );
@@ -298,9 +307,10 @@ specifies the DNS zone to be updated.
 
 sub question {
 	my @qr = @{shift->{question}};
+	return @qr;
 }
 
-sub zone {&question}
+sub zone { return &question }
 
 
 =head2 answer, pre, prerequisite
@@ -318,10 +328,11 @@ not preexist.
 
 sub answer {
 	my @rr = @{shift->{answer}};
+	return @rr;
 }
 
-sub pre		 {&answer}
-sub prerequisite {&answer}
+sub pre		 { return &answer }
+sub prerequisite { return &answer }
 
 
 =head2 authority, update
@@ -338,9 +349,10 @@ specifies the RRs or RRsets to be added or deleted.
 
 sub authority {
 	my @rr = @{shift->{authority}};
+	return @rr;
 }
 
-sub update {&authority}
+sub update { return &authority }
 
 
 =head2 additional
@@ -354,6 +366,7 @@ section of the packet.
 
 sub additional {
 	my @rr = @{shift->{additional}};
+	return @rr;
 }
 
 
@@ -366,7 +379,10 @@ using the master file format mandated by RFC1035.
 
 =cut
 
-sub print { print &string; }
+sub print {
+	print &string;
+	return;
+}
 
 
 =head2 string
@@ -381,38 +397,42 @@ sub string {
 	my $self = shift;
 
 	my $header = $self->header;
-	my $update = $header->opcode eq 'UPDATE';
-
 	my $server = $self->{replyfrom};
 	my $length = $self->{replysize};
-	my $string = $server ? ";; Response received from $server ($length octets)\n" : "";
+	my $origin = $server ? ";; Response received from $server ($length octets)\n" : "";
+	my @record = ( "$origin;; HEADER SECTION", $header->string );
 
-	$string .= ";; HEADER SECTION\n" . $header->string;
+	if ( $self->{dso} ) {
+		CORE::push( @record, ";; DSO SECTION" );
+		foreach ( @{$self->{dso}} ) {
+			my ( $t, $v ) = @$_;
+			CORE::push( @record, pack 'a* A18 a*', ";;\t", dsotypebyval($t), unpack( 'H*', $v ) );
+		}
+		return join "\n", @record, "\n";
+	}
 
-	my $question = $update ? 'ZONE' : 'QUESTION';
-	my @question = map $_->string, $self->question;
+	my @section  = $header->opcode eq 'UPDATE' ? qw(ZONE PREREQUISITE UPDATE) : qw(QUESTION ANSWER AUTHORITY);
+	my @question = $self->question;
 	my $qdcount  = scalar @question;
 	my $qds	     = $qdcount != 1 ? 's' : '';
-	$string .= join "\n;; ", "\n;; $question SECTION ($qdcount record$qds)", @question;
+	CORE::push( @record, ";; $section[0] SECTION ($qdcount record$qds)", map { ';; ' . $_->string } @question );
 
-	my $answer = $update ? 'PREREQUISITE' : 'ANSWER';
-	my @answer  = map $_->string, $self->answer;
+	my @answer  = $self->answer;
 	my $ancount = scalar @answer;
 	my $ans	    = $ancount != 1 ? 's' : '';
-	$string .= join "\n", "\n\n;; $answer SECTION ($ancount record$ans)", @answer;
+	CORE::push( @record, "\n;; $section[1] SECTION ($ancount record$ans)", map { $_->string } @answer );
 
-	my $authority = $update ? 'UPDATE' : 'AUTHORITY';
-	my @authority = map $_->string, $self->authority;
+	my @authority = $self->authority;
 	my $nscount   = scalar @authority;
 	my $nss	      = $nscount != 1 ? 's' : '';
-	$string .= join "\n", "\n\n;; $authority SECTION ($nscount record$nss)", @authority;
+	CORE::push( @record, "\n;; $section[2] SECTION ($nscount record$nss)", map { $_->string } @authority );
 
-	my @additional = map $_->string, $self->additional;
+	my @additional = $self->additional;
 	my $arcount    = scalar @additional;
 	my $ars	       = $arcount != 1 ? 's' : '';
-	$string .= join "\n", "\n\n;; ADDITIONAL SECTION ($arcount record$ars)", @additional;
+	CORE::push( @record, "\n;; ADDITIONAL SECTION ($arcount record$ars)", map { $_->string } @additional );
 
-	return "$string\n\n";
+	return join "\n", @record, "\n";
 }
 
 
@@ -429,10 +449,10 @@ sub from {
 	my $self = shift;
 
 	$self->{replyfrom} = shift if scalar @_;
-	$self->{replyfrom};
+	return $self->{replyfrom};
 }
 
-sub answerfrom { &from; }					# uncoverable pod
+sub answerfrom { return &from; }				# uncoverable pod
 
 
 =head2 size
@@ -446,10 +466,10 @@ nameserver.  This method will return undef for user-created packets
 =cut
 
 sub size {
-	shift->{replysize};
+	return shift->{replysize};
 }
 
-sub answersize { &size; }					# uncoverable pod
+sub answersize { return &size; }				# uncoverable pod
 
 
 =head2 push
@@ -472,7 +492,7 @@ Section names may be abbreviated to the first three characters.
 sub push {
 	my $self = shift;
 	my $list = $self->_section(shift);
-	CORE::push( @$list, grep ref($_), @_ );
+	return CORE::push( @$list, grep { ref($_) } @_ );
 }
 
 
@@ -497,11 +517,11 @@ Section names may be abbreviated to the first three characters.
 sub unique_push {
 	my $self = shift;
 	my $list = $self->_section(shift);
-	my @rr	 = grep ref($_), @_;
+	my @rr	 = grep { ref($_) } @_;
 
 	my %unique = map { ( bless( {%$_, ttl => 0}, ref $_ )->canonical => $_ ) } @rr, @$list;
 
-	scalar( @$list = values %unique );
+	return scalar( @$list = values %unique );
 }
 
 
@@ -518,7 +538,7 @@ Removes a single RR from the specified section of the packet.
 sub pop {
 	my $self = shift;
 	my $list = $self->_section(shift);
-	CORE::pop(@$list);
+	return CORE::pop(@$list);
 }
 
 
@@ -534,7 +554,7 @@ sub _section {				## returns array reference for section
 	my $self = shift;
 	my $name = shift;
 	my $list = $_section{unpack 'a3', $name} || $name;
-	$self->{$list} ||= [];
+	return $self->{$list} ||= [];
 }
 
 
@@ -609,15 +629,13 @@ does not support the suppressed signature scheme described in RFC2845.
 sub sign_tsig {
 	my $self = shift;
 
-	eval {
+	return eval {
 		local $SIG{__DIE__};
 		require Net::DNS::RR::TSIG;
 		my $tsig = Net::DNS::RR::TSIG->create(@_);
 		$self->push( 'additional' => $tsig );
 		return $tsig;
-	} || do {
-		croak "$@\nTSIG: unable to sign packet";
-	};
+	} || return croak "$@\nTSIG: unable to sign packet";
 }
 
 
@@ -663,7 +681,7 @@ The requisite cryptographic components are not integrated into
 Net::DNS but reside in the Net::DNS::SEC distribution available
 from CPAN.
 
-    $update = new Net::DNS::Update('example.com');
+    $update = Net::DNS::Update->new('example.com');
     $update->push( update => rr_add('foo.example.com A 10.1.2.3'));
     $update->sign_sig0('Kexample.com+003+25317.private');
 
@@ -683,7 +701,7 @@ sub sign_sig0 {
 	my $self = shift;
 	my $karg = shift;
 
-	eval {
+	return eval {
 		local $SIG{__DIE__};
 
 		my $sig0;
@@ -697,9 +715,7 @@ sub sign_sig0 {
 
 		$self->push( 'additional' => $sig0 );
 		return $sig0;
-	} || do {
-		croak "$@\nSIG0: unable to sign packet";
-	};
+	} || return croak "$@\nSIG0: unable to sign packet";
 }
 
 
@@ -716,10 +732,10 @@ sub sigrr {
 	my $self = shift;
 
 	my ($sig) = reverse $self->additional;
-	return undef unless $sig;
+	return unless $sig;
 	return $sig if $sig->type eq 'TSIG';
 	return $sig if $sig->type eq 'SIG';
-	return undef;
+	return;
 }
 
 
@@ -769,7 +785,7 @@ sub truncate {
 
 	my $tc;
 	my $hash = {};
-	foreach my $section ( map $self->{$_}, qw(question answer authority) ) {
+	foreach my $section ( map { $self->{$_} } qw(question answer authority) ) {
 		my @list;
 		foreach my $item (@$section) {
 			my $component = $item->encode( length $data, $hash );
@@ -785,7 +801,7 @@ sub truncate {
 
 	my %rrset;
 	my @order;
-	foreach my $item ( grep ref($_) ne ref($sigrr), $self->additional ) {
+	foreach my $item ( grep { ref($_) ne ref($sigrr) } $self->additional ) {
 		my $name  = $item->{owner}->canonical;
 		my $class = $item->{class} || 0;
 		my $key	  = pack 'nna*', $class, $item->{type}, $name;
@@ -812,8 +828,8 @@ sub truncate {
 	$self->{'additional'} = \@list;
 
 	my @part = qw(question answer authority additional);
-	my @size = map scalar( @{$self->{$_}} ), @part;
-	pack 'n6 a*', $self->header->id, $self->{status}, @size, substr( $data, HEADER_LENGTH );
+	my @size = map { scalar @{$self->{$_}} } @part;
+	return pack 'n6 a*', $self->header->id, $self->{status}, @size, substr( $data, HEADER_LENGTH );
 }
 
 
@@ -824,6 +840,7 @@ sub dump {				## print internal data structure
 	local $Data::Dumper::Maxdepth = $Data::Dumper::Maxdepth || 3;
 	local $Data::Dumper::Sortkeys = $Data::Dumper::Sortkeys || 1;
 	print Data::Dumper::Dumper(@_);
+	return;
 }
 
 
@@ -839,7 +856,7 @@ Portions Copyright (c)2002-2004 Chris Reinhardt.
 
 Portions Copyright (c)2002-2009 Olaf Kolkman
 
-Portions Copyright (c)2007-2015 Dick Franks
+Portions Copyright (c)2007-2019 Dick Franks
 
 All rights reserved.
 
