@@ -1,30 +1,31 @@
 package Net::DNS::RR::OPT;
 
-#
-# $Id: OPT.pm 1779 2020-05-11 09:11:17Z willem $
-#
-our $VERSION = (qw$LastChangedRevision: 1779 $)[1];
-
-
 use strict;
 use warnings;
+our $VERSION = (qw$Id: OPT.pm 1812 2020-10-07 18:09:53Z willem $)[2];
+
 use base qw(Net::DNS::RR);
 
+
 =head1 NAME
 
 Net::DNS::RR::OPT - DNS OPT resource record
 
 =cut
 
-
 use integer;
 
 use Carp;
-use Net::DNS::Parameters;
+use Net::DNS::Parameters qw(:rcode :ednsoption);
 
 use constant CLASS_TTL_RDLENGTH => length pack 'n N n', (0) x 3;
 
-use constant OPT => typebyname qw(OPT);
+use constant OPT => Net::DNS::Parameters::typebyname qw(OPT);
+
+require Net::DNS::DomainName;
+require Net::DNS::RR::A;
+require Net::DNS::RR::AAAA;
+require Net::DNS::Text;
 
 
 sub _decode_rdata {			## decode rdata from wire-format octet string
@@ -44,6 +45,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 		$self->{option}{$code} = $value;
 		$offset += $length + 4;
 	}
+	return;
 }
 
 
@@ -51,7 +53,7 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $option = $self->{option} || {};
-	join '', map pack( 'nna*', $_, length $option->{$_}, $option->{$_} ), keys %$option;
+	return join '', map { pack( 'nna*', $_, length $option->{$_}, $option->{$_} ) } keys %$option;
 }
 
 
@@ -61,7 +63,7 @@ sub encode {				## overide RR method
 	my $data = $self->_encode_rdata;
 	my $size = $self->size;
 	my @xttl = ( $self->rcode >> 4, $self->version, $self->flags );
-	pack 'C n n C2n n a*', 0, OPT, $size, @xttl, length($data), $data;
+	return pack 'C n n C2n n a*', 0, OPT, $size, @xttl, length($data), $data;
 }
 
 
@@ -72,7 +74,7 @@ sub string {				## overide RR method
 	my $flags  = sprintf '%04x', $self->flags;
 	my $rcode  = $self->rcode;
 	my $size   = $self->size;
-	my @option = map join( "\n;;\t\t\t\t", $self->_format_option($_) ), $self->options;
+	my @option = map { join( "\n;;\t\t\t\t", $self->_format_option($_) ) } $self->options;
 	my @format = join "\n;;\t\t", @option;
 
 	$rcode = 0 if $rcode < 16;				# weird: 1 .. 15 not EDNS codes!!
@@ -94,15 +96,15 @@ QQ
 sub class {				## overide RR method
 	my $self = shift;
 	$self->_deprecate(qq[please use "size()"]);
-	$self->size(@_);
+	return $self->size(@_);
 }
 
 sub ttl {				## overide RR method
 	my $self = shift;
 	$self->_deprecate(qq[please use "flags()" or "rcode()"]);
-	my @rcode = map unpack( 'C',   pack 'N', $_ ), @_;
-	my @flags = map unpack( 'x2n', pack 'N', $_ ), @_;
-	pack 'C2n', $self->rcode(@rcode), $self->version, $self->flags(@flags);
+	my @rcode = map { unpack( 'C',	 pack 'N', $_ ) } @_;
+	my @flags = map { unpack( 'x2n', pack 'N', $_ ) } @_;
+	return pack 'C2n', $self->rcode(@rcode), $self->version, $self->flags(@flags);
 }
 
 
@@ -110,7 +112,7 @@ sub version {
 	my $self = shift;
 
 	$self->{version} = 0 + shift if scalar @_;
-	$self->{version} || 0;
+	return $self->{version} || 0;
 }
 
 
@@ -126,14 +128,14 @@ sub rcode {
 	return $self->{rcode} || 0 unless scalar @_;
 	delete $self->{rdlength};				# (ab)used to signal incomplete value
 	my $val = shift || 0;
-	$self->{rcode} = $val < 16 ? 0 : $val;			# discard non-EDNS rcodes 1 .. 15
+	return $self->{rcode} = $val < 16 ? 0 : $val;		# discard non-EDNS rcodes 1 .. 15
 }
 
 
 sub flags {
 	my $self = shift;
-	return $self->{flags} || 0 unless scalar @_;
-	$self->{flags} = shift;
+	$self->{flags} = shift if scalar @_;
+	return $self->{flags} || 0;
 }
 
 
@@ -141,13 +143,14 @@ sub options {
 	my ($self) = @_;
 	my $option = $self->{option} || {};
 	my @option = sort { $a <=> $b } keys %$option;
+	return @option;
 }
 
 sub option {
 	my $self   = shift;
 	my $number = ednsoptionbyname(shift);
 	return $self->_get_option($number) unless scalar @_;
-	$self->_set_option( $number, @_ );
+	return $self->_set_option( $number, @_ );
 }
 
 
@@ -162,7 +165,7 @@ sub _format_option {
 	my $defined = length($payload) && $package->can('_image');
 	my @element = $defined ? eval { $package->_image($payload) } : unpack 'H*', $payload;
 	my $protect = pop(@element);
-	Net::DNS::RR::_wrap( "$option\t=> (", map( "$_,", @element ), "$protect )" );
+	return Net::DNS::RR::_wrap( "$option\t=> (", map( {"$_,"} @element ), "$protect )" );
 }
 
 
@@ -172,11 +175,11 @@ sub _get_option {
 	my $options = $self->{option} || {};
 	my $payload = $options->{$number};
 	return $payload unless wantarray;
-	return () unless $payload;
+	return ()	unless $payload;
 	my $package = join '::', __PACKAGE__, ednsoptionbyval($number);
 	$package =~ s/-/_/g;
 	return ( 'OPTION-DATA' => $payload ) unless $package->can('_decompose');
-	my @payload = eval { $package->_decompose($payload) };
+	return eval { $package->_decompose($payload) };
 }
 
 
@@ -203,31 +206,32 @@ sub _set_option {
 			}
 		}
 	}
-	$options->{$number} = $value;
+	return $options->{$number} = $value;
 }
 
 
 sub _specified {
 	my $self = shift;
-	my @spec = grep $self->{$_}, qw(size flags rcode option);
-	scalar @spec;
+	return scalar grep { $self->{$_} } qw(size flags rcode option);
 }
 
 
 ########################################
 
+## no critic ProhibitMultiplePackages
 package Net::DNS::RR::OPT::DAU;					# RFC6975
 
 sub _compose {
-	my ( $class, @argument ) = @_;
-	pack 'C*', @argument;
+	shift;
+	return pack 'C*', @_;
 }
 
 sub _decompose {
 	my @payload = unpack 'C*', $_[1];
+	return @payload;
 }
 
-sub _image { &_decompose; }
+sub _image { return &_decompose; }
 
 
 package Net::DNS::RR::OPT::DHU;					# RFC6975
@@ -238,24 +242,22 @@ our @ISA = qw(Net::DNS::RR::OPT::DAU);
 
 
 package Net::DNS::RR::OPT::CLIENT_SUBNET;			# RFC7871
-use Net::DNS::RR::A;
-use Net::DNS::RR::AAAA;
 
-my %family = qw(1 Net::DNS::RR::A	2 Net::DNS::RR::AAAA);
+my %family = qw(0 Net::DNS::RR::AAAA	1 Net::DNS::RR::A	2 Net::DNS::RR::AAAA);
 my @field8 = qw(FAMILY SOURCE-PREFIX-LENGTH SCOPE-PREFIX-LENGTH ADDRESS);
 
 sub _compose {
-	my ( $class, %argument ) = @_;
+	my ( $class, %argument ) = ( map( ( $_ => 0 ), @field8 ), @_ );
 	my $address = bless( {}, $family{$argument{FAMILY}} )->address( $argument{ADDRESS} );
 	my $bitmask = $argument{'SOURCE-PREFIX-LENGTH'};
-	pack "a* B$bitmask", pack( 'nC2', map $_ || 0, @argument{@field8} ), unpack 'B*', $address;
+	pack "a* B$bitmask", pack( 'nC2', @argument{@field8} ), unpack 'B*', $address;
 }
 
 sub _decompose {
 	my %hash;
 	@hash{@field8} = unpack 'nC2a*', $_[1];
 	$hash{ADDRESS} = bless( {address => $hash{ADDRESS}}, $family{$hash{FAMILY}} )->address;
-	my @payload = map { ( $_ => $hash{$_} ) } @field8;
+	my @payload = map( ( $_ => $hash{$_} ), @field8 );
 }
 
 sub _image {
@@ -278,24 +280,29 @@ sub _decompose {
 sub _image { join ' => ', &_decompose; }
 
 
-package Net::DNS::RR::OPT::COOKIE;				# RFC7873
+package Net::DNS::RR::OPT::COOKIE;				# draft-ietf-dnsop-server-cookies
 
-my @field10 = qw(CLIENT-COOKIE SERVER-COOKIE);
+my @field10 = qw(VERSION RESERVED TIMESTAMP HASH);
 
 sub _compose {
-	my ( $class, %argument ) = @_;
-	pack 'a8 a*', map $_ || '', @argument{@field10};
+	my ( $class, %argument ) = ( VERSION => 1, RESERVED => '', @_ );
+	return pack 'a8', $argument{'CLIENT-COOKIE'} if $argument{'CLIENT-COOKIE'};
+	pack 'Ca3Na*', map $_, @argument{@field10};
 }
 
 sub _decompose {
+	my ( $class, $argument ) = @_;
+	return ( 'CLIENT-COOKIE', $argument ) unless length($argument) > 8;
 	my %hash;
-	@hash{@field10} = unpack 'a8 a*', $_[1];
-	my @payload = map { ( $_ => $hash{$_} ) } @field10;
+	@hash{@field10} = unpack 'Ca3Na*', $argument;
+	my @payload = map( ( $_ => $hash{$_} ), @field10 );
 }
 
 sub _image {
-	my %hash  = &_decompose;
-	my @image = map join( ' => ', $_, unpack 'H*', $hash{$_} ), @field10;
+	my %hash = &_decompose;
+	return unpack 'H*', $hash{'CLIENT-COOKIE'} if $hash{'CLIENT-COOKIE'};
+	for (qw(RESERVED HASH)) { $hash{$_} = unpack 'H*', $hash{$_} }
+	my @image = map "$_ => $hash{$_}", @field10;
 }
 
 
@@ -329,7 +336,6 @@ sub _image { join ' => ', &_decompose; }
 
 
 package Net::DNS::RR::OPT::CHAIN;				# RFC7901
-use Net::DNS::DomainName;
 
 sub _compose {
 	my ( $class, %argument ) = @_;
@@ -349,8 +355,8 @@ sub _image { join ' => ', &_decompose; }
 package Net::DNS::RR::OPT::KEY_TAG;				# RFC8145
 
 sub _compose {
-	my ( $class, @argument ) = @_;
-	pack 'n*', @argument;
+	shift;
+	pack 'n*', @_;
 }
 
 sub _decompose {
@@ -361,19 +367,21 @@ sub _image { &_decompose; }
 
 
 package Net::DNS::RR::OPT::EXTENDED_ERROR;			# draft-ietf-dnsop-extended-error
-use Net::DNS::Text;
 
 my @field15 = qw(INFO-CODE EXTRA-TEXT);
 
 sub _compose {
-	my ( $class, %argument ) = @_;
-	my ( $ic, $et ) = map $_ || '', @argument{@field15};
-	pack 'na*', $ic, Net::DNS::Text->new($et)->raw;
+	my ( $class, %argument ) = ( 'INFO-CODE' => 0, 'EXTRA-TEXT' => '', @_ );
+	my ( $code,  $text )	 = @argument{@field15};
+	pack 'na*', $code, Net::DNS::Text->new($text)->raw;
 }
 
 sub _decompose {
-	my ( $ic, $et ) = unpack 'na*', $_[1];
-	my @payload = ( 'INFO-CODE' => $ic, 'EXTRA-TEXT' => Net::DNS::Text->decode( \$et, 0, length $et )->value );
+	my ( $code, $text ) = unpack 'na*', $_[1];
+	my @payload = (
+		'INFO-CODE'  => $code,
+		'EXTRA-TEXT' => Net::DNS::Text->decode( \$text, 0, length $text )->value
+		);
 }
 
 sub _image {
@@ -389,13 +397,13 @@ __END__
 =head1 SYNOPSIS
 
     use Net::DNS;
-    $packet = new Net::DNS::Packet( ... );
+    $packet = Net::DNS::Packet->new( ... );
 
     $packet->header->do(1);			# extended flag
 
     $packet->edns->size(1280);			# UDP payload size
 
-    $packet->edns->option( COOKIE => $cookie );
+    $packet->edns->option( COOKIE => 'rawbytes' );
 
     $packet->edns->print;
 
@@ -403,10 +411,10 @@ __END__
     ;;	    flags:  8000
     ;;	    rcode:  NOERROR
     ;;	    size:   1280
-    ;;	    option: DAU	   => ( 8, 10, 13, 14, 15, 16 )
+    ;;	    option: COOKIE => ( 7261776279746573 )
+    ;;		    DAU	   => ( 8, 10, 13, 14, 15, 16 )
     ;;		    DHU	   => ( 1, 2, 4 )
-    ;;		    COOKIE => ( CLIENT-COOKIE => 7261776279746573,
-    ;;				SERVER-COOKIE =>  )
+    ;;		    EXTENDED-ERROR => ( INFO-CODE => 123, EXTRA-TEXT =>	 )
 
 
 =head1 DESCRIPTION
@@ -469,8 +477,8 @@ header.
 
 	$octets = $packet->edns->option($option_code);
 
-	$packet->edns->option( COOKIE => $cookie );
-	$packet->edns->option( 10     => $cookie );
+	$packet->edns->option( COOKIE => $octets );
+	$packet->edns->option( 10     => $octets );
 
 When called in a list context, options() returns a list of option codes
 found in the OPT record.
@@ -480,27 +488,24 @@ option() returns the uninterpreted octet string
 corresponding to the specified option.
 The method returns undef if the specified option is absent.
 
-Options can be added or replaced by providing the (name => string) pair.
+Options can be added or replaced by providing the (name => value) pair.
 The option is deleted if the value is undefined.
 
 
 When option() is called in a list context with a single argument,
-the returned array provides a structured interpretation
+the returned values provide a structured interpretation
 appropriate to the specified option.
 
-For the example above:
+For example:
 
-	%hash = $packet->edns->option(10);
-
-	%hash = (
-		'CLIENT-COOKIE' => 'rawbytes',
-		'SERVER-COOKIE' => ''
-		);
+	@algorithms = $packet->edns->option('DAU');
 
 
-For some options, an array is more appropriate:
+For some options, a hash table is more convenient:
 
-	@algorithms = $packet->edns->option(6);
+	%hash_table = $packet->edns->option(15);
+	$info_code  = $hash_table{'INFO-CODE'};
+	$extra_text = $hash_table{'EXTRA-TEXT'};
 
 
 Similar forms of array or hash syntax may be used to construct the
@@ -508,14 +513,14 @@ option value:
 
 	$packet->edns->option( DHU => [1, 2, 4] );
 
-	$packet->edns->option( COOKIE => {'CLIENT-COOKIE' => $cookie} );
+	$packet->edns->option( EXPIRE => {'EXPIRE-TIMER' => 604800} );
 
 
 =head1 COPYRIGHT
 
 Copyright (c)2001,2002 RIPE NCC.  Author Olaf M. Kolkman.
 
-Portions Copyright (c)2012,2017-2019 Dick Franks.
+Portions Copyright (c)2012,2017-2020 Dick Franks.
 
 All rights reserved.
 
