@@ -2,7 +2,7 @@ package Net::DNS::Resolver::Base;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: Base.pm 1812 2020-10-07 18:09:53Z willem $)[2];
+our $VERSION = (qw$Id: Base.pm 1814 2020-10-14 21:49:16Z willem $)[2];
 
 
 #
@@ -261,22 +261,23 @@ sub print {
 }
 
 
-sub domain {
-	my $self   = shift;
-	my ($head) = $self->searchlist(@_);
-	my @list   = grep {defined} $head;
-	return wantarray ? @list : "@list";
-}
-
 sub searchlist {
-	my $self = shift;
+	my ( $self, @domain ) = @_;
 	$self = $self->_defaults unless ref($self);
 
-	return $self->{searchlist} = [@_] unless defined wantarray;
-	$self->{searchlist} = [@_] if scalar @_;
+	if ( !defined(wantarray) or scalar(@domain) ) {
+		foreach (@domain) { $_ = Net::DNS::Domain->new($_)->name }
+		$self->{searchlist} = [@domain];
+	}
+
 	return ( @{$self->{searchlist}} );
 }
 
+sub domain {
+	my ($head) = &searchlist;
+	return wantarray ? ( grep {defined} $head ) : $head;
+}
+
 
 sub nameservers {
 	my $self = shift;
